// Test of the FunctionNote class
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sat Mar  2 22:32:12 PST 1996
// Last Modified: Wed Apr 17 20:41:33 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/glissfunction.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include "main.cc"

EventBuffer eventBuffer(2000);	// EventBuffer holding up to 2000 events


/***************** Beginning of user functions ****************************/

//////////////////////////////
//
// UpGlissNoteFunction -- creates upward chromatic glissandos.  To be
//      used with the FunctionNote class; Notes are generated one at a
//      time in the EventBuffer from a FunctionNote.
//

static void UpGlissNoteFunction(Note* p) {
   static Note tn;		// temporary note before placing in buffer

   tn.setOnDur(t_time, p->getOffTime()); // off time holds dur
   tn.play(p->getKey()+1, (int)(p->getVel()*drum.p1ps()), p->getChan());

   p->setKeyno(tn.getKey());
   p->setOnTime(p->getOnTime() + p->getOffTime());  // OffTime stores duration

   eventBuffer.insert(&tn);

   if (p->getKey() >= C7) p->off();
}



//////////////////////////////
//
// DownGlissNoteFunction -- creates downward chromatic glissandos.  To be
//      used with the FunctionNote class; Notes are generated one at a
//      time in the EventBuffer.
//

static void DownGlissNoteFunction(Note* p) {
   static Note tn;              // temporary note before placing in buffer

   tn.setOnDur(t_time, p->getOffTime()); // off time holds dur
   tn.play(p->getKey()-1, (int)(p->getVel()*drum.p1ps()), p->getChan());

   p->setKeyno(tn.getKey());
   p->setOnTime(p->getOnTime() + p->getOffTime());  // OffTime stores duration

   eventBuffer.insert(&tn);

   if (p->getKey() <= A0) p->off();
}


/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|    Test of the FunctionNote class using glissandos.         |\n"
   << "|    Baton 1 creates upward glissandos, and Baton 2 creates   |\n"
   << "|    downward glissandos.   -- Craig Stuart Sapp, 2 Mar 1996  |\n"
   << "---------------------------------------------------------------\n"
   ;
}



void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1: glissando up chromatically                          \n"
   << ">    x1t: starting note of the glissando                       \n"
   << ">    y1t: speed of glissando                                   \n"
   << ">    w1t: key velocity                                         \n"
   << "> Baton 2: glissando down chromatically                        \n"
   << ">    x2t: starting note of the glissando                       \n"
   << ">    y2t: speed of glissando                                   \n"
   << ">    w2t: key velocity                                         \n"
   << ">                                                              \n"
   << ">    p1p: control attack velocity of up glisses                \n"
   << ">    p2p: control attack velocity of down glisses              \n"
   << ">    p3p: control timbre of glissandos at trigger time         \n"
   << ">   +b14: command mode                                         \n"
   << ">   +b15: this help screen                                     \n"
   << ">                                                              \n"
   ;
}


/*---------------- Initialization functions ------------------------------*/

void initialization(void) { 
   eventBuffer.setPollRate(10);
   drum.setPollRate(25);
   description(); 
}


/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { 
   eventBuffer.checkPoll();
   drum.checkPollPots();
}

/*---------------- Trigger response functions ----------------------------*/

void stick1trig(void) { 
   static Note tn;	// a Temporary Note for copying into eventBuffer

   // setting the fields of the function note, fields are defined in function
   tn.setStatus(FUNCTION_NOTE);
   tn.setTag((char*)&UpGlissNoteFunction);
   tn.setKeyno(drum.x1ts(A0, C7));
   tn.setVelocity(drum.w1ts(20, 100));

   // off time of note holds duration:
   tn.setOnOff(t_time, drum.y1ts(eventBuffer.getPollRate(), 1000));

   eventBuffer.insert(&tn);

   cout << "UP:   StartKey = " << drum.x1ts(A0, C7)
        <<     "\tVelocity = " << drum.w1ts(20,100)
        <<     "\tRate = "     << drum.y1ts(eventBuffer.getPollRate(), 1000)
        << endl;
}



void stick2trig(void) {
   static Note tn;

   // setting the fields of the function note, fields are defined in function
   tn.setStatus(FUNCTION_NOTE);
   tn.setTag((char*)&DownGlissNoteFunction);
   tn.setKeyno(drum.x2ts(A0, C7));
   tn.setVelocity(drum.w2ts(A0, C7));

   // off time of note holds duration:
   tn.setOnOff(t_time, drum.y2ts(eventBuffer.getPollRate(), 1000));

   eventBuffer.insert(&tn);

   cout << "DOWN: StartKey = " << drum.x2ts(A0, C7)
        <<     "\tVelocity = " << drum.w2ts(20, 100)
        <<     "\tRate = "     << drum.y2ts(eventBuffer.getPollRate(), 1000)
        << endl;
}



void b14plustrig(void) { cout << "B14+ triggered\n"; commandmode(); }
void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }

/*---------------- Poll response functions -------------------------------*/

void stick1pollresponse(void) { }
void stick2pollresponse(void) { }

void potspollresponse(void) { 
   if (drum.p1p != drum.p1po) {
      cout << "\t\tDown vel env = " << drum.p1ps() << endl;
   }

   if (drum.p2p != drum.p2po) {
      cout << "\t\t\tUp vel env = " << drum.p2ps() << endl;
   }

   if (drum.p3ps(0,63) != drum.p3pos(0,63)) {
      midi.pc(CH_1, drum.p3ps(0,63));
      cout << "\t\t\t\tTimbre = " << drum.p3ps(0, 63) << endl;
   }
}


/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

