// Test of the FunctionNote class
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sat Mar  2 22:32:12 PST 1996
// Last Modified: Sat Apr 27 18:06:56 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/glissfunction2.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include "main.cc"

EventBuffer eventBuffer(2000);	// EventBuffer holding up to 2000 events

class glissParameters { 
   public:
      short direction;
      short step;
      short key;
      short vel;
      int   speed;
      glissParameters(void) { 
         direction = 0; key = A0; vel = 127; speed = 100; step = 1;
      }
};

glissParameters fred;
      

/***************** Beginning of user functions ****************************/

//////////////////////////////
//
// GlissNoteFunction 
//

#define PARAMS ((glissParameters*)((FunctionNote*)note)->parameters())

static void GlissNoteFunction(Note* note) {
   static Note tempnote;

   PARAMS->key += PARAMS->direction * PARAMS->step;
   if (PARAMS->key > C7 || PARAMS->key < A0) {
      note->off();
      PARAMS->direction = 0;
      return;
   }

   PARAMS->vel += 30;
   if (PARAMS->vel > 127) PARAMS->vel = 127;

   if (PARAMS->speed > 100) 
      tempnote.setOnDur(t_time, PARAMS->speed);
   else
      tempnote.setOnDur(t_time, 100);
   tempnote.play(PARAMS->key,(int)(PARAMS->vel * drum.p1ps()), note->getChan());
   eventBuffer.insert(&tempnote);

   note->setOnTime(note->getOnTime() + PARAMS->speed); 
}



/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|    Test of the FunctionNote class using glissandos.         |\n"
   << "|    Baton 1 creates upward glissandos, and Baton 2 creates   |\n"
   << "|    downward glissandos.   -- Craig Stuart Sapp, 2 Mar 1996  |\n"
   << "---------------------------------------------------------------\n"
   ;
}



void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1: glissando up chromatically                          \n"
   << ">    x1t: starting note of the glissando                       \n"
   << ">    y1t: speed of glissando                                   \n"
   << ">    w1t: key velocity                                         \n"
   << "> Baton 2: glissando down chromatically                        \n"
   << ">    x2t: starting note of the glissando                       \n"
   << ">    y2t: speed of glissando                                   \n"
   << ">    w2t: key velocity                                         \n"
   << ">                                                              \n"
   << ">    p1p: control attack velocity of up glisses                \n"
   << ">    p2p: control attack velocity of down glisses              \n"
   << ">    p3p: control timbre of glissandos at trigger time         \n"
   << ">   +b14: command mode                                         \n"
   << ">   +b15: this help screen                                     \n"
   << ">                                                              \n"
   ;
}


/*---------------- Initialization functions ------------------------------*/

void initialization(void) { 
   eventBuffer.setPollRate(10);
   drum.setPollRate(25);
   description(); 
}


/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { 
   eventBuffer.checkPoll();
   drum.checkPollPots();
}

/*---------------- Trigger response functions ----------------------------*/

void stick1trig(void) { 
   fred.vel = drum.w1ts(20, 100);
//   fred.speed = drum.y1ts(eventBuffer.getPollRate(), 1000);
//   fred.speed = drum.y1ts(0, 500);

   cout << "UP:   StartKey = " << fred.key
        <<     "\tVelocity = " << fred.vel
        <<     "\tRate = "     << fred.speed
        << endl;

   if (fred.direction == 0) {
      fred.key = drum.x1ts(A0, C7);
      fred.direction = 1;
      static FunctionNote tempnote;  
      tempnote.setFunction(&GlissNoteFunction);
      tempnote.setParameters(&fred);
      tempnote.setOnOff(t_time, 0);
      eventBuffer.insert(&tempnote);
   } else if (fred.direction == -1) {
      fred.direction = 1;
   }
}


void stick2trig(void) { 
   fred.vel = drum.w2ts(20, 100);
   fred.speed = drum.y2ts(eventBuffer.getPollRate(), 1000);

   cout << "Down:   StartKey = " << fred.key
        <<       "\tVelocity = " << fred.vel
        <<       "\tRate = "     << fred.speed
        << endl;

   if (fred.direction == 0) {
      fred.key = drum.x2ts(A0, C7);
      fred.direction = -1;
      static FunctionNote tempnote;
      tempnote.setFunction(&GlissNoteFunction);
      tempnote.setParameters(&fred);
      tempnote.setOnOff(t_time, 0);
      eventBuffer.insert(&tempnote);
   } else if (fred.direction == 1) {
      fred.direction = -1;
   }
}


void b14plustrig(void) { cout << "B14+ triggered\n"; commandmode(); }
void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }

/*---------------- Poll response functions -------------------------------*/

void stick1pollresponse(void) { }
void stick2pollresponse(void) { }

void potspollresponse(void) { 
   if (drum.p1ps(0,30) != drum.p1pos(0,30)) {
      cout << "\t\tvel env = " << drum.p1ps() << endl;
   }

   if (drum.p2ps(0,12) != drum.p2pos(0,12)) {
      fred.step = drum.p2ps(0,12);
      cout << "\tstep = " << drum.p2ps(0,12) << endl;
   }
   
   if (drum.p3ps(0,63) != drum.p3pos(0,63)) {
      midi.pc(CH_1, drum.p3ps(0,63));
      cout << "\t\t\t\tTimbre = " << drum.p3ps(0, 63) << endl;
   }
}


/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

