// hotcold.cc -- The game of hot and cold for the radio drum.
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Tue Mar 19 13:35:34 PST 1996
// Last Modified: Mon Apr  8 21:35:04 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/hotcold.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include <math.h>
#include "main.cc"


/***************** Beginning of user function *****************************/

Note  xVoice(CH_1, 0);              // voice for x-axis: piano timbre
Note  yVoice(CH_2, 10);             // voice for x-axis: vibraphone timbre
Note  zVoice(CH_3, 12);             // voice for x-axis: ???   timbre

int  xrand, yrand, zrand;           // random point in baton space
int  status = OFF;                  // test whether to play notes
long timer;                         // time how long it takes to find point

int nextXtime, nextYtime, nextZtime;
int oldXnote, oldYnote, oldZnote;

/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|    Hit the drum with a baton to start a search for a random |\n"
   << "| point.  Notes stop playing when the 3-D point is found.     |\n"
   << "| You are hot when the pitches rise; cold when pitch drops.   |\n"
   << "| Each axis has a voice associated with itself.               |\n"
   << "| Tempos of each axis controlled by baton 2.                  |\n"
   << "|                          -- Craig Stuart Sapp, 21 Apr 1996  |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1:                                                     \n"
   << ">   trig: Start the hot/cold game                              \n"
   << ">    x1p: X-axis note -- higher if warmer, lower if colder     \n"
   << ">    y1p: Y-axis note -- higher if warmer, lower if colder     \n"
   << ">    z1p: Z-axis note -- higher if warmer, lower if colder     \n"
   << "> Baton 2:                                                     \n"
   << ">    x2p: X-axis tempo.                                        \n"
   << ">    y2p: Y-axis tempo.                                        \n"
   << ">    z2p: Z-axis tempo.                                        \n"
   << ">                                                              \n"
   << ">    p1p: Attack velocity of x direction voice                 \n"
   << ">    p2p: Attack velocity of y direction voice                 \n"
   << ">    p3p: Attack velocity of z direction voice                 \n"
   << ">   +b14: Command mode                                         \n"
   << ">   +b15: This help screen                                     \n"
   << ">                                                              \n"
   ;
}


/*---------------- Initialization algorithms -----------------------------*/

void initialization(void) { 
   drum.setPollRate(20);       // poll rate of baton set to 20 milliseconds
   description();
   xVoice.sendTimbre();
   yVoice.sendTimbre();
   zVoice.sendTimbre();
   nextXtime = nextYtime = nextZtime = 0;
   oldXnote = oldYnote = oldZnote = 0;
}


/*---------------- Main loop algorithms ----------------------------------*/

void mainloopalgorithms(void) { 
   drum.checkPollAll();
   if (status == ON) {

      // play the notes for each axis if a new note space is entered:
      if (t_time >= nextXtime && drum.x1ps(A0, C7) != oldXnote) {
         oldXnote = drum.x1ps(A0, C7);
         xVoice.play(C7-abs(xrand - oldXnote), drum.p1p);
         nextXtime = t_time + 80 + 3 * drum.x2p;
      } 
      if (t_time >= nextYtime && drum.y1ps(A0, C7) != oldYnote)  {
         oldYnote = drum.y1ps(A0, C7);
         yVoice.play(C7-abs(yrand - oldYnote), drum.p2p);
         nextYtime = t_time + 80 + 3 * drum.y2p;
      }
      if (t_time >= nextZtime && drum.z1ps(A0, C7) != oldZnote)  {
         oldZnote = drum.z1ps(A0, C7);
         zVoice.play(C7-abs(zrand - oldZnote), drum.p3p);
         nextZtime = t_time + 80 + 3 * drum.z2p;
      }


      // if the 3-D point was found, turn off the notes and print time:
      if (   abs(drum.x1ps(A0, C7) - xrand) < 10  &&
             abs(drum.y1ps(A0, C7) - yrand) < 10  &&
             abs(drum.z1ps(A0, C7) - zrand) < 10       ) {
         status = OFF;
         xVoice.off();
         yVoice.off();
         zVoice.off();
         cout << "Time to find point: " << (t_time-timer)/1000.0 << " sec.\n";
      }
   }
}


/*---------------- Trigger response functions ----------------------------*/

void stick1trig(void) { 
   // if not already searching for a point, choose a new point and start timer:
   if (status == OFF) {
      status = ON;
      xrand = rand() % (C7-A0) + A0;
      yrand = rand() % (C7-A0) + A0;
      zrand = (rand() % 44) + 44;        // limit z-axis close to drum surface
      timer = t_time;
      cout << "random point chosen\n"; 
   }
}

void stick2trig(void) { }

void b14plustrig(void) { 
   status = OFF;
   xVoice.off();
   yVoice.off();
   zVoice.off();
   commandmode(); 
}

void b15plustrig(void) { programhelp(); }

/*---------------- Poll response algorithms ------------------------------*/

void stick1pollresponse(void) { }
void stick2pollresponse(void) { }
void potspollresponse(void) { }

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

