// major.cc -- Test of simultaneous triggering with chords
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sat Mar  9 13:23:13 PST 1996
// Last Modified: Tue Apr 23 00:20:01 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/major.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include "main.cc"

Note  bass;                         // submediant note
Note  tenor;                        // tonic note
Note  alto;                         // mediant note
Note  soprano;                      // dominant note
Note  quintus;                      // subtonic note

const int HitHard = 80;
const int HitMedium = 50;

/***************** Beginning of user functions ****************************/


int makeDigit(int x, int y) {
   static const int digitHolder[5][2] =                     // 1 2 3 4 5
                           {6,1,7,2,8,3,9,4,0,5};           // 6 7 8 9 0
   return digitHolder[x][y];
}

int makeKeyno(void) {
   return 10 * makeDigit(drum.x1ts(0,4), drum.y1ts(0,1)) +
           1 * makeDigit(drum.x2ts(0,4), drum.y2ts(0,1));
}


/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|    A major triad will be played if both batons are          |\n"
   << "|    triggered at the same time.  The root of the triad       |\n"
   << "|    is controlled by the paragraph below.  If baton 1        |\n"
   << "|    is hit hard, a minor third below the root will be        |\n"
   << "|    played; if hit very hard, a major third below will be    |\n"
   << "|    played.  In a similar manner, if baton 2 is hit hard,    |\n"
   << "|    notes will be played above the fifth of the triad.       |\n"
   << "|                                                             |\n"
   << "|         The radio drum is divided into 10 locations:        |\n"
   << "|    5 divisions along the x direction and 2 divisions along  |\n"
   << "|    the y direction.  Batons  control the key number of the  |\n"
   << "|    MIDI note, Baton 1 controlling the tens digits of the    |\n"
   << "|    key number, and Baton 2 controlling the ones digit of    |\n"
   << "|    the key number. Digits are arranged on the radio drum as:|\n"
   << "|              1  2  3  4  5                                  |\n"
   << "|              6  7  8  9  0                                  |\n"
   << "|                          -- Craig Stuart Sapp, 10 Mar 1996  |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout

   << ">                                                              \n"
   << "> Baton 1: Tens digit of keyno to play                         \n"
   << ">    x1t: Five divisions                                       \n"
   << ">    y1t: Two divisions                                        \n"
   << ">    w1t: Averages with w2t for key velocity of some notes.    \n"
   << "> Baton 2: Ones digit of keyno to play                         \n"
   << ">    x2t: Five divisions                                       \n"
   << ">    y2t: Two divisions                                        \n"
   << ">    w2t: Averages with w1t for key velocity of some notes.    \n"
   << ">                                                              \n"
   << ">    p1p: Channel Number to play chord on.                     \n"
   << ">    p2p: Timber to play chord with.                           \n"
   << ">   +b14: Command mode.                                        \n"
   << ">   +b15: This help screen.                                    \n"
   << ">                                                              \n"
   ;
}

/*---------------- Initialization functions ------------------------------*/

void initialization(void) { 
   description(); 
   drum.setPollRate(20);		// poll rate of baton set to 20 ms
}

/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { drum.checkPollPots(); }

/*---------------- Trigger response functions ----------------------------*/

void ActIfTogether(void) {
   if (drum.togetherQ(60)) {
      int triggerKey, channel, average;

      triggerKey = makeKeyno();
      channel = drum.p1ps(0, 15);
      average = (drum.w1t + drum.w2t)/2;

      cout << "Keyno = " << triggerKey
           << "\t w1t = " << drum.w1t << "\tw2t = " << drum.w2t << endl;

      //            key           vel           chan
      tenor.play   (triggerKey,   drum.w1t,     channel);
      alto.play    (triggerKey+4, average,      channel);
      soprano.play (triggerKey+7, drum.w2t,     channel);

      // determine bass note:
      if (drum.w1t > HitMedium && triggerKey > 3) {
         if (drum.w1t > HitHard) {
            bass.play(triggerKey-4, average, channel);
         } else {
            bass.play(triggerKey-3, average, channel);
         }
      } else {
            bass.off();
      }

      // determine quintus note:
      if (drum.w2t > HitMedium) {
         if (drum.w2t > HitHard) {
            quintus.play(triggerKey+11, average, channel);
         } else {
            quintus.play(triggerKey+10, average, channel);
         }
      } else {
         quintus.off();
      }

      drum.clearTriggers();
   }
}


void stick1trig(void) {
   cout << "Baton 1 triggered." << endl;
   drum.setT1(t_time);
   ActIfTogether();
}

void stick2trig(void) {
   cout << "Baton 2 triggered." << endl;
   drum.setT2(t_time);
   ActIfTogether();
}

void b14plustrig(void) { cout << "B14+ triggered\n"; commandmode(); }
void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }

/*---------------- Poll response functions -------------------------------*/

void stick1pollresponse(void) { }
void stick2pollresponse(void) { }
void potspollresponse(void) { 
   if (drum.cp1p(1,16)) {
      cout << "\t\t\t    Channel = " << drum.p1ps(1,16) << endl;
   }

   if (drum.cp2p(0,63)) {
      cout << "\t     Timbre = " << drum.p2ps(0,63) << endl;
      tenor.sendTimbre(drum.p2ps(0,63));
   }
}

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

