//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sun May 12 18:02:36 GMT-0700 1996
// Last Modified: Sun May 12 18:02:36 GMT-0700 1996
// Filename:      /user/c/craig/lang/c/drum/examples/octave-unison.cc
// Syntax:        C++ (Gnu C++ Complier: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include "main.cc"

EventBuffer eventBuffer(2000);

NoteLoop octave_loop[2];

/***************** Beginning of user functions ****************************/


class LoopParameters {
   public:
      float velenv;
      int status;
      int lastKey;
      int currentKey;
      int loopnum;
      LoopParameters(void) 
         { velenv = 1;  
           status = 0;
           lastKey = 60; 
           currentKey = 60;
           loopnum = 0; }
};

LoopParameters loopParameters[2];

#define PARAMS ((LoopParameters*)((FunctionNote*)note)->parameters())

static void loopingFunction(Note * note) {
   static Note tempnote;
   if (PARAMS->status == -1) {
      note->off();
   } else if (PARAMS->status == 0) {
      note->setOnTime(t_time + 1000);
   } else {
      PARAMS->lastKey = PARAMS->currentKey;
      PARAMS->currentKey = octave_loop[PARAMS->loopnum].nextKey();
      octave_loop[PARAMS->loopnum].setCurKey((PARAMS->lastKey + PARAMS->currentKey)/2);
      tempnote.setChan(note->getChannel());
      tempnote.setKey(PARAMS->currentKey);
      tempnote.setVel((int)(octave_loop[PARAMS->loopnum].nextVel() * PARAMS->velenv));
      tempnote.setOnDur(t_time, octave_loop[PARAMS->loopnum].nextDur());
      note->setOnTime(note->getOnTime() + octave_loop[PARAMS->loopnum].curDur());
      tempnote.play();
      eventBuffer.insert(&tempnote);
   }
}


/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|                                                             |\n"
   << "|                                                             |\n"
   << "|                                                             |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1:                                                     \n"
   << ">    x1t:                                                      \n"
   << ">    y1t:                                                      \n"
   << ">    w1t:                                                      \n"
   << ">    x1p:                                                      \n"
   << ">    y1p:                                                      \n"
   << ">    z1p:                                                      \n"
   << "> Baton 2:                                                     \n"
   << ">    x2t:                                                      \n"
   << ">    y2t:                                                      \n"
   << ">    w2t:                                                      \n"
   << ">    x2p:                                                      \n"
   << ">    y2p:                                                      \n"
   << ">    z2p:                                                      \n"
   << ">                                                              \n"
   << ">    p1p:                                                      \n"
   << ">    p2p:                                                      \n"
   << ">    p3p:                                                      \n"
   << ">   +b14: Command mode.                                        \n"
   << ">   +b15: This help screen.                                    \n"
   << ">                                                              \n"
   ;
}

/*---------------- Initialization functions ------------------------------*/

void makeOctaveLoop(int num, int degree) {
   char keys[6];
   for (int i=0; i<6; i++) {
      keys[i] = 24 + i * 12 + degree;
   }
   char vels[10] = {100, 123, 80, 30, 45, 20, 100, 127, 90, 110};
   long durs[7] = {100, 200, 300, 40, 50, 60, 70};
  
   octave_loop[num].setKeys(keys, 6);
   octave_loop[num].setVels(vels, 10);
   octave_loop[num].setDurs(durs, 7);
}

void initialization(void) { 
   description(); 
   drum.setPollRate(40);
   eventBuffer.setPollRate(5);
   makeOctaveLoop(0, 0);
   makeOctaveLoop(1, 6);
   FunctionNote tempnote;
   tempnote.setFunction(&loopingFunction);
   tempnote.setParameters(&loopParameters[0]);
   eventBuffer.insert(&tempnote);
   loopParameters[1].loopnum = 1;
   tempnote.setParameters(&loopParameters[1]);
}

/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { 
   drum.checkPollBatons();
   eventBuffer.checkPoll();
}

/*---------------- Trigger response functions ----------------------------*/

void stick1trig(void) { 
   makeOctaveLoop(0, drum.x1ts(0, 11));      
}
void stick2trig(void) { 
   makeOctaveLoop(1, drum.x2ts(0, 11));      
}
void b14plustrig(void) { cout << "B14+ triggered\n"; commandmode(); }
void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }

/*---------------- Poll response functions -------------------------------*/

void stick1pollresponse(void) { 
   loopParameters[0].status = 1;
   loopParameters[0].velenv = drum.y1ps();
}

void stick2pollresponse(void) { 
   loopParameters[1].status = 1;
   loopParameters[1].velenv = drum.y2ps();
}

void potspollresponse(void) { }

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

