// soundfile.c -- shows how to play a single soundfile at a time.  
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sat Apr 20 20:38:19 PDT 1996
// Last Modified: Sun Apr 21 14:45:27 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/soundfile.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include "main.cc"
#include <string.h>                       // for copying soundfile names
#include <fstream.h>                      // for checking for valid filenames
extern "C" {
#include <sound/utilsound.h>              // for SNDPlaySoundfile function
}

const int MaxStringSize = 2048;
char soundfile[MaxStringSize] = {0};    // name of soundfile for stick1trig
char inputname[MaxStringSize] = {0};    // test name of soundfile
ifstream file;                          // for checking for vaild filenames

/***************** Beginning of user functions ****************************/

/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|    Baton 1 trigger plays a soundfile.  Baton 2 trigger      |\n"
   << "| reads in a new soundfile name to play with Baton 1 trigger. |\n"
   << "|                         -- Craig Stuart Sapp, 21 April 1996 |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout
   << ">                                                                \n"
   << "> Baton 1:                                                       \n"
   << ">   trig: Play specified soundfile:                              \n"
   << ">         default soundfile: /NextLibrary/Sounds/Poit.snd .      \n"
   << "> Baton 2:                                                       \n"
   << ">   trig: Get the name of a soundfile to play from the keyboard. \n"
   << ">         Will not accept non-existant filenames.                \n"
   << ">                                                                \n"
   << ">   +b14: Command mode.                                          \n"
   << ">   +b15: This help screen.                                      \n"
   << ">                                                                \n"
   ;
}


/*---------------- Initialization functions ------------------------------*/

void initialization(void) { 
   description(); 
   strncpy(soundfile, "/NextLibrary/Sounds/Poit.snd", MaxStringSize);
}


/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { }


/*---------------- Trigger response functions ----------------------------*/

void stick1trig(void) { 
   cout << "Stick 1 triggered\n"; 
   SNDPlaySoundfile(soundfile, /* Priorty = */ 1);
}


void stick2trig(void) { 
   cout << "Stick 2 triggered\n\n";
   cout << "\tEnter a new soundfile name to play: ";
   cin  >> inputname;
   file.open(inputname);
   if (file.is_open()) {
      file.close();
      strncpy(soundfile, inputname, MaxStringSize);
      cout << "\tNew soundfile is: " << soundfile << endl << endl;
      inputname[0] = '\0';
   } else {
      cout << "\tFile was not changed." << endl << endl;
   }
}


void b14plustrig(void) { cout << "B14+ triggered\n"; commandmode(); }
void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }


/*---------------- Poll response functions -------------------------------*/

void stick1pollresponse(void) { }
void stick2pollresponse(void) { }
void potspollresponse(void) { }

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

