// soundfile2.cc -- Example of triggering soundfiles on different areas of 
//	the radio drum.
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sat Apr 20 20:38:19 PDT 1996
// Last Modified: Sun Apr 21 14:51:27 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/soundfile2.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include "main.cc"

#include <math.h>				// for rand() function
extern "C" {
#include <sound/utilsound.h>			// for SNDPlaySoundfile function
}

const int numSounds = 15;
char* sounds[numSounds] = {
       "/NextLibrary/Sounds/Basso.snd",
       "/NextLibrary/Sounds/Block.snd",
       "/NextLibrary/Sounds/Bonk.snd",
       "/NextLibrary/Sounds/Bullfrog.snd",
       "/NextLibrary/Sounds/Cricket.snd",
       "/NextLibrary/Sounds/Crow.snd",
       "/NextLibrary/Sounds/Drip.snd",
       "/NextLibrary/Sounds/Frog.snd",
       "/NextLibrary/Sounds/Funk.snd",
       "/NextLibrary/Sounds/Glass.snd",
       "/NextLibrary/Sounds/Ni.snd",
       "/NextLibrary/Sounds/Ping.snd",
       "/NextLibrary/Sounds/Poit.snd",
       "/NextLibrary/Sounds/Pop.snd",
       "/NextLibrary/Sounds/Rooster.snd"
};


/***************** Beginning of user functions ****************************/

/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|    Play a random Soundfile.                                 |\n"
   << "|                         -- Craig Stuart Sapp, 21 April 1996 |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1:                                                     \n"
   << ">    x1t:                                                      \n"
   << ">    y1t:                                                      \n"
   << ">    w1t:                                                      \n"
   << ">    x1p:                                                      \n"
   << ">    y1p:                                                      \n"
   << ">    z1p:                                                      \n"
   << "> Baton 2:                                                     \n"
   << ">    x2t:                                                      \n"
   << ">    y2t:                                                      \n"
   << ">    w2t:                                                      \n"
   << ">    x2p:                                                      \n"
   << ">    y2p:                                                      \n"
   << ">    z2p:                                                      \n"
   << ">                                                              \n"
   << ">    p1p:                                                      \n"
   << ">    p2p:                                                      \n"
   << ">    p3p:                                                      \n"
   << ">   +b14: Command mode.                                        \n"
   << ">   +b15: This help screen.                                    \n"
   << ">                                                              \n"
   ;
}

/*---------------- Initialization functions ------------------------------*/

void initialization(void) { 
   description(); 
}

/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { }

/*---------------- Trigger response functions ----------------------------*/

void stick1trig(void) { 
   cout << "Baton 1 triggered.\t"; 
   char* name = sounds[rand() % numSounds];
   cout << "Playing sound: " << name << endl;
   SNDPlaySoundfile(name, 1);
}

void stick2trig(void) { 
   cout << "Baton 2 triggered.\t"; 
   char* name = sounds[rand() % numSounds];
   cout << "Playing sound: " << name << endl;
   SNDPlaySoundfile(name, 1);
}

void b14plustrig(void) { cout << "B14+ triggered\n"; commandmode(); }
void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }

/*---------------- Poll response functions -------------------------------*/

void stick1pollresponse(void) { }
void stick2pollresponse(void) { }
void potspollresponse(void) { }

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

